@@ -72,21 +72,17 @@ class OrderPoint(ModelSQL, ModelView):
     @classmethod
     def __setup__(cls):
         super(OrderPoint, cls).__setup__()
-        cls._constraints += [
-            ('check_concurrent_internal', 'concurrent_internal_op'),
-            ('check_uniqueness', 'unique_op'),
-            ]
         cls._sql_constraints += [
             ('check_max_qty_greater_min_qty',
                 'CHECK(max_quantity >= min_quantity)',
                 'Maximal quantity must be bigger than Minimal quantity'),
             ]
         cls._error_messages.update({
-                'unique_op': 'Only one order point is allowed '\
-                    'for each product-location pair.',
-                'concurrent_internal_op': 'You can not define ' \
-                    'two order points on the same product ' \
-                    'with opposite locations.',
+                'unique_op': ('Only one order point is allowed '
+                    'for each product-location pair.'),
+                'concurrent_internal_op': ('You can not define '
+                    'two order points on the same product '
+                    'with opposite locations.'),
                 })
 
     @classmethod
@@ -120,6 +116,12 @@ class OrderPoint(ModelSQL, ModelView):
     def get_unit_digits(self, name):
         return self.product.default_uom.digits
 
+    @classmethod
+    def validate(cls, orderpoints):
+        super(OrderPoint, cls).validate(orderpoints)
+        cls.check_concurrent_internal(orderpoints)
+        cls.check_uniqueness(orderpoints)
+
     @classmethod
     def check_concurrent_internal(cls, orders):
         """
@@ -132,7 +134,7 @@ class OrderPoint(ModelSQL, ModelView):
                 ('type', '=', 'internal'),
                 ])
         if not internals:
-            return True
+            return
 
         query = ['OR']
         for op in internals:
@@ -142,7 +144,8 @@ class OrderPoint(ModelSQL, ModelView):
                    ('company', '=', op.company.id),
                    ('type', '=', 'internal')]
             query.append(arg)
-        return not bool(cls.search(query))
+        if cls.search(query):
+            cls.raise_user_error('concurrent_internal_op')
 
     @staticmethod
     def _type2field(type=None):
@@ -150,7 +153,7 @@ class OrderPoint(ModelSQL, ModelView):
             'purchase': 'warehouse_location',
             'internal': 'storage_location',
             }
-        if type == None:
+        if type is None:
             return t2f
         else:
             return t2f[type]
@@ -172,7 +175,8 @@ class OrderPoint(ModelSQL, ModelView):
                 ('company', '=', op.company.id),
                 ]
             query.append(arg)
-        return not bool(cls.search(query))
+        if cls.search(query):
+            cls.raise_user_error('unique_op')
 
     def get_rec_name(self, name):
         return "%s@%s" % (self.product.name, self.location.name)
