@@ -16,15 +16,23 @@ __all__ = ['PurchaseRequest',
     'CreatePurchaseRequestStart', 'CreatePurchaseRequest',
     'CreatePurchaseAskTerm', 'CreatePurchaseAskParty', 'CreatePurchase']
 
+STATES = {
+    'readonly': Eval('state') != 'draft',
+    }
+DEPENDS = ['state']
+
 
 class PurchaseRequest(ModelSQL, ModelView):
     'Purchase Request'
     __name__ = 'purchase.request'
     product = fields.Many2One('product.product', 'Product', required=True,
         select=True, readonly=True, domain=[('purchasable', '=', True)])
-    party = fields.Many2One('party.party', 'Party',  select=True)
-    quantity = fields.Float('Quantity', required=True)
-    uom = fields.Many2One('product.uom', 'UOM', required=True, select=True)
+    party = fields.Many2One('party.party', 'Party', select=True, states=STATES,
+        depends=DEPENDS)
+    quantity = fields.Float('Quantity', required=True, states=STATES,
+        depends=DEPENDS)
+    uom = fields.Many2One('product.uom', 'UOM', required=True, select=True,
+        states=STATES, depends=DEPENDS)
     computed_quantity = fields.Float('Computed Quantity', readonly=True)
     computed_uom = fields.Many2One('product.uom', 'Computed UOM',
         readonly=True)
@@ -63,8 +71,8 @@ class PurchaseRequest(ModelSQL, ModelView):
         super(PurchaseRequest, cls).__setup__()
         cls._order[0] = ('id', 'DESC')
         cls._error_messages.update({
-                'create_request': 'Purchase requests are only created ' \
-                    'by the system.',
+                'create_request': ('Purchase requests are only created '
+                    'by the system.'),
                 })
         cls._sql_constraints += [
             ('check_purchase_request_quantity', 'CHECK(quantity > 0)',
@@ -129,7 +137,7 @@ class PurchaseRequest(ModelSQL, ModelView):
         return res
 
     @classmethod
-    def generate_requests(cls):
+    def generate_requests(cls, products=None):
         """
         For each product compute the purchase request that must be
         create today to meet product outputs.
@@ -157,13 +165,14 @@ class PurchaseRequest(ModelSQL, ModelView):
                 (order_point.warehouse_location.id, order_point.product.id)
                 ] = order_point
 
-        # fetch goods and assets
-        # ordered by ids to speedup reduce_ids in products_by_location
-        products = Product.search([
-                ('type', 'in', ['goods', 'assets']),
-                ('consumable', '=', False),
-                ('purchasable', '=', True),
-                ], order=[('id', 'ASC')])
+        if products is None:
+            # fetch goods and assets
+            # ordered by ids to speedup reduce_ids in products_by_location
+            products = Product.search([
+                    ('type', 'in', ['goods', 'assets']),
+                    ('consumable', '=', False),
+                    ('purchasable', '=', True),
+                    ], order=[('id', 'ASC')])
         product_ids = [p.id for p in products]
         #aggregate product by minimum supply date
         date2products = {}
@@ -192,7 +201,7 @@ class PurchaseRequest(ModelSQL, ModelView):
 
                     for product in products[i:i + cursor.IN_MAX]:
                         shortage_date, product_quantity = shortages[product.id]
-                        if shortage_date == None or product_quantity == None:
+                        if shortage_date is None or product_quantity is None:
                             continue
                         order_point = product2ops.get(
                             (warehouse_id, product.id))
@@ -202,6 +211,14 @@ class PurchaseRequest(ModelSQL, ModelView):
                             company, order_point)
                         new_requests.append(request)
 
+        # delete purchase requests without a purchase line
+        products = set(products)
+        reqs = cls.search([
+                ('purchase_line', '=', None),
+                ('origin', 'like', 'stock.order_point,%'),
+                ])
+        reqs = [r for r in reqs if r.product in products]
+        cls.delete(reqs)
         new_requests = cls.compare_requests(new_requests)
 
         cls.create_requests(new_requests)
@@ -220,15 +237,9 @@ class PurchaseRequest(ModelSQL, ModelView):
         Compare new_requests with already existing request to avoid
         to re-create existing requests.
         """
-        # delete purchase request without a purchase line
         pool = Pool()
         Uom = pool.get('product.uom')
         Request = pool.get('purchase.request')
-        reqs = Request.search([
-            ('purchase_line', '=', None),
-            ('origin', 'like', 'stock.order_point,%'),
-            ])
-        Request.delete(reqs)
 
         requests = Request.search([
                 ('purchase_line.moves', '=', None),
@@ -256,8 +267,8 @@ class PurchaseRequest(ModelSQL, ModelView):
             existing_req.setdefault(
                 (request.product.id, request.warehouse.id),
                 []).append({
-                        'supply_date': \
-                            request.supply_date or datetime.date.max,
+                        'supply_date': (
+                            request.supply_date or datetime.date.max),
                         'quantity': quantity,
                         'uom': uom,
                         })
@@ -323,21 +334,13 @@ class PurchaseRequest(ModelSQL, ModelView):
         Date = Pool().get('ir.date')
 
         supplier = None
-        timedelta = datetime.timedelta.max
         today = Date.today()
         for product_supplier in product.product_suppliers:
             supply_date = product_supplier.compute_supply_date(date=today)
-            sup_timedelta = date - supply_date
-            if not supplier:
+            timedelta = date - supply_date
+            if not supplier and timedelta >= datetime.timedelta(0):
                 supplier = product_supplier.party
-                timedelta = sup_timedelta
-                continue
-
-            if timedelta < datetime.timedelta(0) \
-                    and (sup_timedelta >= datetime.timedelta(0) \
-                    or sup_timedelta > timedelta):
-                supplier = product_supplier.party
-                timedelta = sup_timedelta
+                break
 
         if supplier:
             purchase_date = product_supplier.compute_purchase_date(date)
@@ -440,11 +443,12 @@ class PurchaseRequest(ModelSQL, ModelView):
             for x in product_ids)
 
     @classmethod
-    def create(cls, vals):
-        for field_name in ('product', 'quantity', 'uom', 'company'):
-            if not vals.get(field_name):
-                cls.raise_user_error('create_request')
-        return super(PurchaseRequest, cls).create(vals)
+    def create(cls, vlist):
+        for vals in vlist:
+            for field_name in ('product', 'quantity', 'uom', 'company'):
+                if not vals.get(field_name):
+                    cls.raise_user_error('create_request')
+        return super(PurchaseRequest, cls).create(vlist)
 
 
 class CreatePurchaseRequestStart(ModelView):
@@ -460,7 +464,7 @@ class CreatePurchaseRequest(Wizard):
             Button('Cancel', 'end', 'tryton-cancel'),
             Button('Create', 'create_', 'tryton-ok', default=True),
             ])
-    create_ = StateAction('stock_supply.act_purchase_request_form_draft')
+    create_ = StateAction('stock_supply.act_purchase_request_form')
 
     def do_create_(self, action):
         PurchaseRequest = Pool().get('purchase.request')
@@ -507,10 +511,10 @@ class CreatePurchase(Wizard):
     def __setup__(cls):
         super(CreatePurchase, cls).__setup__()
         cls._error_messages.update({
-            'missing_price': 'Purchase price is missing for product: %s ' \
-                '(id: %s)!',
-            'please_update': 'This price is necessary for creating purchase.'
-            })
+                'missing_price': 'Purchase price is missing for product "%s".',
+                'please_update': ('This price is necessary for creating '
+                    'purchases.'),
+                })
 
     def default_ask_party(self, fields):
         Request = Pool().get('purchase.request')
@@ -589,8 +593,8 @@ class CreatePurchase(Wizard):
             with Transaction().set_context(
                     company=self.ask_term.company.id):
                 Party.write([self.ask_term.party], {
-                        'supplier_payment_term': \
-                            self.ask_term.payment_term.id,
+                        'supplier_payment_term': (
+                            self.ask_term.payment_term.id),
                         })
             self.ask_term.payment_term = None
             self.ask_term.party = None
@@ -668,8 +672,8 @@ class CreatePurchase(Wizard):
                 Decimal(1) / 10 ** Line.unit_price.digits[1])
 
         if product_price is None:
-            cls.raise_user_error('missing_price', (request.product.name,
-                    request.product.id), 'please_update')
+            cls.raise_user_error('missing_price', (request.product.rec_name,),
+                'please_update')
         line.unit_price = product_price
 
         taxes = []
